!-----------------------------------------------------------------------
!
!  NOKIA 620 HSF handset (at least -2AC)
!
!  HSF mods by
!  Pavel Lajsner (OK2UCX) http://www.qsl.net/ok0ns
!
!  ORIGINAL (HSN2) BY:
!  Fri Mar 10 20:21:03 EET 2000
!  Juha Nurmela
!
!  uPD7810 CPU
!  uPD7228 LCD
!  audio HSIC
!  and other misc fluff
!
#define VERSION "HSF2 0.2--10.04.00"
!                12345678--12345678
!
!----------------------------------------------------------------------
!
#define MBUSID "HSN2"
!----------------------------------------------------------------------
!
!  We SEND each keypress in separate packet.
!
!    0 "ANY1" "HSN2" "M" "." \4         e.g. make of '1'
!    0 "ANY1" "HSN2" "R" "." \4         e.g. repeat of '1'
!    0 "ANY1" "HSN2" "B" "." \4         e.g. break of '1' (no key pressed)
!    0 "ANY1" "HSN2" "H" "[nf]" \4      e.g. hook oN/oFf
!
!  We accept following packets.
!
!    0 "HSN2" "...." "f" "........" \4  e.g. display "  433500"
!    0 "HSN2" "...." "s" "...?" \4      e.g. display report 99 (optional [sS])
!    0 "HSN2" "...." "v" "." \4         e.g. volume setting 0..7
!    0 "HSN2" "...." "l" "s?t?" \4      e.g. lights SERV, ALPHA, BACKLIGHT...
!    0 "HSN2" "...." "m" "s?e?" \4      e.g. mute speaker and/or earphone
!    0 "HSN2" "...." "b" "# 5 5 * " \4  e.g. play DTMF, R is special for 1750
!                                       last chr keeps playing (use ' ' to end)
!
!----------------------------------------------------------------------
!
!  portpins

! PA, all outputs (VIEWCONT initially 0=high angle, could be input too)


SCK      = 0x01

! PB, POWERSW and BUSY inputs

POWERSW  = 0x01 ! input, "0" if "OFF" pressed
POWERAMP = 0x02 ! audio PA supply
ASTOP    = 0x04 ! general audio supply shutdown (STOP) (1 stopped?)
LCD_CS   = 0x08 ! LCD enable (uPD7228 LCD chip select)
INPCONT	 = 0x70 ! 3 bit bus for HF mic gain (?)
BUSY     = 0x80 ! /BUSY from LCD

! PC, HOOKIND input

NMIMASK  = 0x04 ! hold /NMI "1" when set
OFFHOOK  = 0x08 ! input
SI       = 0x10 ! to LCD
NETFREE  = 0x20 ! input (with hw mode)
VIEWCONT = 0x40 ! 3-way, 0=high angle, input=low angle, 1=; for lcd contrast
LCD_CD   = 0x80 ! command/data for LCD

! Analog inputs AN0...3 in select mode

!----------------------------------------------------------------------
!
!  keyboard PA common rails
!

KBD_R0    = 0x02 | 1
KBD_R1    = 0x04 | 1
KBD_R2    = 0x08 | 1
KBD_R3    = 0x10 | 1
KBD_R4    = 0x20 | 1
KBD_R5    = 0x40 | 1
KBD_R6    = 0x80 | 1

!----------------------------------------------------------------------

HSIC_ADDR0 = 0x0400 ! HSIC address #0
HSIC_ADDR1 = 0x0500 ! HSIC address #1
HSIC_ADDR2 = 0x0600 ! HSIC address #2
HSIC_ADDR3 = 0x0700 ! HSIC address #3

!----------------------------------------------------------------------
! hsic register definitions

! hsic 0:
IMICMUTE = 0x01	! mutes IMIC -> TX path
EXTMICMUTE = 0x02	! mutes EXTMIC -> TX path
HFMICMUTE = 0x04	! mutes HFMIC -> TX path
MODSEL = 0x07	! mask for all modulation mutes
MODVOLUME = 0x18 ! 00, 01, 10 (three different levels), 11 causes total reset
ESPKMUTE = 0x40	! mutes ESPK somehow
ISPKMUTE = 0x80	! mutes ISPK somehow

! hsic 1
MODMUTE = 0x01 	! completely mutes modulation (TX) path
SPKRXMUTE = 0x02	! mutes RX -> SPK path
SOMEMUTE = 0x04	! hmmm, must be "1" for RX->SPK part to be active
VOLUME = 0x78	! 4 bits(!) TX path volume

! hsic 2
TONEMASK = 0x1F ! bits 0..3 tone #, DTMF mask also included
DTMFMODE = 0x10	! 0 DTMF, 1 single tone
MODMODE = 0x20	! 0 dtmf/tone -> TX, 1 selected (hsic0) audio source -> TX
SPKMODE = 0x40	! 0 DTMF -> SPK, 1 RX -> SPK

! hsic 3
LED_ROAM  = 0x01
LED_SERV  = 0x02
LED_CALL  = 0x04
LED_HF    = 0x08
LED_ON    = 0x10
LED_MFT   = 0x20
BACKLIGHT = 0x40

!----------------------------------------------------------------------
!
!  LCD symbol characters are
!  F C N   < - >  T   A L P H A   key
!  We will use characters in FCN and ALPHA groups as a single symbols.
!
ICON_FCN   = 0x0002 ! (column_offset << 8) | (strlen(fcn) - 1)
ICON_LEFT  = 0x0305 ! < somewhere in this area (0x03 .. 0x08)
ICON_RIGHT = 0x0D00 ! >
ICON_MINUS = 0x0E00 ! -
ICON_T     = 0x1e13 ! somewhere in this area (0x1e .. 0x32)
ICON_ALPHA = 0x0f04
ICON_KEY   = 0x1409 ! somewhere in this area

! 0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF012
! FCN<<<<<<    >-ALPHAkkkkkkkkkkTTTTTTTTTTTTTTTTTTTTT
!
!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  external RAM 8192 bytes
!  stack is located at the end of the ram.
!  Remember, workarea only 256 bytes (V.wa)
!  DO NOT CHANGE V from workarea !

	.data
	.org 0xE000
ram:
workarea: ! XXX cook up some macros to give "safety" with LO(var)

latches:     .rs 8   ; ASSERT(LO(latches) == 0)

lcd_cursor:  .rs 1

mute_state:  .rs 1
volume:      .rs 1
mute_flags:  .rs 1   ! bit 0x01 for ear off, 0x02 for spkr off

kbd_bounce:  .rs 1   ! ticks to 0 whenever nonzero
kbd_repeat:  .rs 1   ! ticks to 255 whenever nonzero
kbd_flags:   .rs 1
 SHIFT   = 1
 ALPHA   = 2
 KEYLOCK = 4

cur_key:     .rs 1
last_pc:     .rs 1

seen_pwr:    .rs 1

middle_row_buffer: .rs 8
bottom_row_buffer: .rs 8

hsic0:	.rs 1
hsic1:	.rs 1
hsic2:	.rs 1
hsic_led:	.rs 1

	ASSERT(. <= workarea + 256)

#include "mbus.asm"

	! 3 kB of stack left. after above include
	.org 0xFFFE
stack:

!----------------------------------------------------------------------
!
!
!
	.text
!--------------------
	.org 0x0000     ! /RESET
	di
	jmp _start

!--------------------
	.org 0x0004     ! /NMI - wakeup!
	reti

!
!  Flags are not tested when entrypoints
!  have only one unmasked source.
!  The flag will be automatically cleared (when vectoring ?).
!
!--------------------
	.org 0x0008     ! INTT1 - 112 Hz (INTT0 not used)
	jmp systick

!--------------------
	.org 0x0010     ! INT1 /INT2 (INT1 is grounded, unused)
	jmp gone_off_hook

!--------------------
	.org 0x0018     ! INTE0 - timer/event counter, NETFREE indication
	jmp netfree

	.org 0x0020     ! INTEIN INTAD - CI/TO edge, A/D complete (unused)
	ei
	reti

	ASSERT(. <= 0x0028)
!--------------------
	.org 0x0028     ! INTST INTSR - serial port (both interrupts unmasked)
	jmp serint

	ASSERT(. <= 0x0060) ! catch overrun
!--------------------
	.org 0x0060     ! SOFTI - software interrupt (might be used)
	reti

!----------------------------------------------------------------------
!
!  Call table
!
#define CALT_REC(x) tab_/**/x = (. - 0x0080) / 2; .word x;
	ASSERT(. <= 0x0080) ! catch overrun
!--------------------
	.org 0x0080
	CALT_REC(latch_0)
	CALT_REC(latch_1)

	ASSERT(. <= 0x00C0) ! End of call table

!----------------------------------------------------------------------
!
!  interrupt functions, no register saved yet
!
gone_off_hook:
	ei
	reti

systick:
	exa
	mov A, [kbd_bounce]
	nei A, 0
	?jr 1f
	sui A, 1
	mov [kbd_bounce], A
1:
	mov A, [kbd_repeat]
	nei A, 0
	?jr 1f
	nei A, 0xFF
	?jr 1f
	adi A, 1
	mov [kbd_repeat], A
1:
	call hsic_update

	exa
	ei
	reti

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  MAIN LOOP
!

mainloop:

	neiw LO(kbd_bounce), 0
	?call scan_keys

	neiw LO(kbd_repeat), 255
	?call repeat_key

	lhld [mbus_rxwp]
	dmov EA, HL
	lhld [mbus_rxrp]
	deq EA, HL
	?call mbus_process

	! hlt

	jmp mainloop

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  STARTUP
!

_start:

	mvi A, 0
	mov MM, A       ! will just use the external RAM
	lxi SP, stack
	mvi V, HI(workarea) ! DO NOT CHANGE V from workarea

	lxi HL, ram
	lxi EA, 0
	mvi A, 0
1:
	stax [HL]+
	deq EA, HL
	?jr 1b

	call cpu_init
	call led_init
	call lcd_init
	call kbd_init
	call vol_set
	call mbus_init
	call hsic_init
	call enable_ints
	ei

 	mvi A, BACKLIGHT
 	call latch_1

	call dtmf_off

	call lcd_middle_row

	lxi HL, version
	call lcd_string

	jmp mainloop

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  CPU configuration

cpu_init:

	mvi PB, LCD_CS | ASTOP | INPCONT
	mvi A, BUSY | POWERSW
	mov MB, A            ! inputs 1

	mvi PA, SCK          ! SCK = 1
	mvi A, 0
	mov MA, A            ! inputs 1

	mvi PC, NMIMASK  ! unmask /nmi later if using powersaves
	mvi A, 0x23                  ! CI,RXD,TXD special
	mov MCC, A
	mvi A, NETFREE | OFFHOOK | 0x02 ! last for RXD
	mov MC, A            ! inputs 1

	mvi ANM, 0x00  ! AN4..7 are used only thru edge flags. 0..3 in scan mode.

	mvi TMM, 0x20  ! TMR0 run from f/12 into F/F, TMR1 run from f/384
	mvi A, 3
	mov TM0, A     ! 921.6 kHz, to generate serial 9600 bd for MBUS
	mvi A, 0xFF
	mov TM1, A     ! to generate 112 Hz systick (will use like 100 Hz)

	mvi A, 0x5E
	mov SML, A     ! 1stop, odd parity, 8bits, x16 clock
	mvi A, 0
	mov SMH, A     ! flush
	mvi A, 0x0C
	mov SMH, A     ! rx enable, tx enable, internal TO clock

	mvi C, 7
1:
	skit INTFST    ! flush tx
	?nop
	skit INTFSR    ! flush rx
	?nop
	skit ER        ! flush rx
	?nop
	mov A, RXB     ! flush rx
	dcr C
	?jr 1b

	mvi A, 0
	mov EOM, A     ! not interested at PWM capabilities

	mvi A, 0x08
	mov ETMM, A    ! clear ECNT at CI falling edge, internal f/12 clock
	lxi EA, 1843
	dmov ETM0, EA  ! 921.6 kHz / ETM0 = 500 Hz, about 20 mbus chars.

	ani PB, ~ASTOP ! un-stop AUDIO (for debug use, to see reset)
	ori PB, POWERAMP ! enable power amplifier
	ret

enable_ints:

	ani MKL, ~0x24 ! INTE0, INTT1
	ani MKH, ~0x06 ! INTST, INTSR

	skit INTFT1
	?nop
	skit INTFE0
	?nop
	skit INTFST
	?nop
	skit INTFSR
	?nop

	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  Audio stuff
!

hsic_init:
	mvi A, 0	! no muting at all (some is done automagically in hsic_update)
	mov [hsic0], A	! audio select
	
	mvi A, 0	! no muting at all, lowest volume
	mov [hsic1], A	! audio select

	mvi A, SPKMODE | SOMEMUTE	! RX -> SPK to be active
	mov [hsic2], A	! audio select
	
	ret

mute_set:
        mov A, [mute_flags]

        oni A, 2
        ?ori PB, POWERAMP
	offi A, 2
        ?ani PB, ~POWERAMP      ! PA required for speaker only
mute_audio:
	push BC
	mov B, [hsic0]
	ori B, ESPKMUTE | ISPKMUTE
        oni A, 1
        ?ani B, ~ISPKMUTE
        oni A, 2
	?ani B, ~ESPKMUTE
	mov [hsic0], B
	pop BC
        ret

vol_set:
	mov A, [volume]
!!vol_any:
!!	eqi A, 0
!!	?jr 1f		! non-zero volume
!!	mvi A, 0x03
!!	call mute_audio	! mute both speakers if zero volume
!!	jmp 2f

1:	push BC
	mov B, [hsic1]
 	ani B, ~VOLUME
	offi A, 0x01
 	?ori B, 0x08
	offi A, 0x02
 	?ori B, 0x10
	offi A, 0x04
 	?ori B, 0x20
	offi A, 0x08
 	?ori B, 0x40
	mov [hsic1], B
	call mute_set	! restore muting
	pop BC

2:	ret

keyclick:
	push VA
!!	mvi A, 9        ! click_vol XXX
!!	eqi A, 0
!!	?call 1f
	call 1f
	pop VA
	ret

1:
!!	call vol_any		!! volume (A) set incl. muting at zero volume

	mvi A, 0x19      ! tone
	call i2c_dtmf

	hlt; hlt; hlt;	! wait, wait, wait ...
	hlt; hlt; hlt;

	call dtmf_off	! no tone
!!	call vol_set
!!	call mute_set
	ret

select_mic:
	push BC
	mov B, [hsic2]
	ori B, MODMODE 		! 1 selected audio source -> TX (no DTMF)
	mov [hsic2], B

	mov B, [hsic0]
	ori B, MODSEL		! mute all

        oni PC, OFFHOOK
        ?ani B, ~HFMICMUTE	! demute HFMIC

        offi PC, OFFHOOK
        ?ani B, ~IMICMUTE	! demute IMIC

	mov [hsic0], B
	pop BC
        ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  OUTPUT LATCHES

latch_0:
	push BC
	xri A, 0xff
	mov C, A
	mov A, [hsic_led]
	ana A, C
	mov [hsic_led], A
	pop BC
	ret

latch_1:
	push BC
	mov C, A
	mov A, [hsic_led]
	ora A, C
	mov [hsic_led], A
	pop BC
	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  LEDS

led_init:
 	mvi A, LED_ON
 	call latch_1
 
 	mvi A, LED_SERV
 	call latch_0
 	mvi A, LED_ROAM
 	call latch_0
 	mvi A, LED_CALL
 	call latch_1       ! CALL until first mbus message for us.
 	mvi A, LED_HF
 	call latch_0
 	mvi A, LED_MFT
 	call latch_0
 
 	mvi A, BACKLIGHT
 	call latch_1
	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  DTMF
!
i2c_dtmf:
	push BC
	mov B, [hsic2]
	ani B, ~SPKMODE ! DTMF -> SPKR now!
	ani B, ~TONEMASK ! mask out the old dtmf tones
	ora A, B	! mask new dtmf tone in
	mov [hsic2], A
	pop BC
	ret

dtmf_off:
        call select_mic

	push BC
	mov B, [hsic2]
	ori B, SPKMODE ! RXaudio -> SPKR now
	mov [hsic2], B
	pop BC

	ret

! for our own keys

dtmf_kbd_table:
	.byte '1', 0x00, '2', 0x04, '3', 0x08, 'F', 0x0C ! DTMF A
	.byte '4', 0x01, '5', 0x05, '6', 0x09, 'C', 0x0D ! DTMF B
	.byte '7', 0x02, '8', 0x06, '9', 0x0A, 'E', 0x0E ! DTMF C
	.byte '*', 0x03, '0', 0x07, '#', 0x0B, 'H', 0x0F ! DTMF D
	.byte 'P', 0x18 ! 04 PTT
	.byte 'N', 0x17 ! 03 NEXT (PREV)
	.byte 'O', 0x16 ! 02 OK (LOCK)
	.byte '+', 0x15 ! 1C +
	.byte 'R', 0x14 ! 1760 Hz
	.byte 'A', 0x13 ! 06 R (ALPHA)
	.byte '?', 0x12 ! 07 MENU (MFT)
	.byte '-', 0x11 ! 1B -
	.byte 'M', 0x10 ! 01 MUTE
	.byte 0, 0

! for common ascii codes

dtmf_chr_table:
	.byte '1', 0x00, '2', 0x04, '3', 0x08, 'F', 0x0C ! DTMF A
	.byte '4', 0x01, '5', 0x05, '6', 0x09, 'C', 0x0D ! DTMF B
	.byte '7', 0x02, '8', 0x06, '9', 0x0A, 'E', 0x0E ! DTMF C
	.byte '*', 0x03, '0', 0x07, '#', 0x0B, 'H', 0x0F ! DTMF D
	.byte 'R', 0x18 ! 1760 Hz
	.byte 0, 0

dtmf_chr:
	push HL
	push BC
	mov C, A
	lxi HL, dtmf_chr_table
	jr 1f

dtmf_kbd:
	push HL
	push BC
	mov C, A
	lxi HL, dtmf_kbd_table
1:
	ldax [HL]+
	nei A, 0
	?jr 1f
	nea A, C
	?jr 1f
	inx HL
	jr 1b
1:
	ldax [HL]
	pop BC
	pop HL
	call i2c_dtmf

	push BC
	mov B, [hsic2]
	ani B, ~MODMODE ! 0: DTMF -> TX audio
	mov [hsic2], B
	pop BC

	ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  LCD
!
!  CA=3 generates the middle 8 character cells and the top 2
!  small character cells [0...9]
!  CA=0 generates the bottom 8 character cells [10...17]

blanks: .asciz "                    " ! 20. enough.

lcd_init_seq_0:
1:
	.byte 0x18 ! - set multiplex mode (SYNC direction ???)
	.byte 0x12 ! - set frame freq
	.byte 0x09 ! - display on
	.byte 0x66 ! - set WRITE mode
	.byte 0x80 ! - datapointer
lcd_init_size_0 = . - 1b

lcd_init_seq_3:
1:
	.byte 0x1A ! - set multiplex mode XXX (SYNC direction ???)
	.byte 0x12 ! - set frame frequency
	.byte 0x09 ! - display on
	.byte 0x66 ! - set WRITE mode
	.byte 0x80 ! - datapointer
lcd_init_size_3 = . - 1b

lcd_init:

	call lcd_command_select_3

	lxi HL, lcd_init_seq_3
	mvi C, lcd_init_size_3
	call lcd_byte_block

	call lcd_data_select_3
	mvi C, 49
1:
	mvi A, 0
	call lcd_byte          ! Empty out totally
	dcr C
	?jr 1b

	call lcd_command_select_0

	lxi HL, lcd_init_seq_0
	mvi C, lcd_init_size_0
	call lcd_byte_block

	call lcd_data_select_0
	mvi C, 49
1:
	mvi A, 0
	call lcd_byte
	dcr C
	?jr 1b

	jmp lcd_middle_row     ! Leave cursor at middle row

lcd_data_mode:
	ani PC,~LCD_CD ! C_D = 0
	call lcd_delay
	ret
lcd_command_mode:
	ori PC,LCD_CD  ! C_D = 1
	call lcd_delay
	ret
lcd_select:
	ani PB, ~LCD_CS ! CS = 0 
	call lcd_delay
	ret
lcd_unselect:
	ori PB, LCD_CS  ! CS = 1
	call lcd_delay
	ori PA, SCK        ! clock sense idle "1" (just in case)
	ret

lcd_data_select_3:
	call lcd_unselect
	call lcd_data_mode
	jr 1f

lcd_command_select_3:
	call lcd_unselect
	call lcd_command_mode
1:
	call lcd_select
	mvi A, 3
	call lcd_byte      ! CA
	ret

lcd_data_select_0:
	call lcd_unselect
	call lcd_data_mode
	jr 1f

lcd_command_select_0:
	call lcd_unselect
	call lcd_command_mode
1:
	call lcd_select
	mvi A, 0
	call lcd_byte      ! CA
	ret

lcd_delay:
	nop
	nop
	ret

lcd_bit:
	ani PA, ~SCK
	sllc A
	?ani PC, ~SI
	skn CY
	?ori PC, SI
	call lcd_delay
	ori PA, SCK
	ret

lcd_byte:
1:
	oni PB, BUSY
	?jr 1b          ! busywait LCD /BUSY

	call lcd_bit
	call lcd_bit
	call lcd_bit
	call lcd_bit

	call lcd_bit
	call lcd_bit
	call lcd_bit
	call lcd_bit

	ret

lcd_string:
1:
	ldax [HL]+
	nei A, 0
	?ret
	call lcd_chr
	jr 1b

! lcd_byte_block is entered in the midst, it also does NOT walk cursor
! (command use)

1:
	ldax [HL]+
	call lcd_byte
lcd_byte_block:
	dcr C
	?jr 1b
	ret

! lcd_block is entered in the midst, it also DOES walk cursor

1:
	ldax [HL]+
	call lcd_chr
lcd_block:
	dcr C
	?jr 1b
	ret

lcd_chr:
	call lcd_byte
	inrw LO(lcd_cursor)
	?nop
	neiw LO(lcd_cursor), 3
	?call lcd_middle_3
	neiw LO(lcd_cursor), 4
	?call lcd_middle_4
	neiw LO(lcd_cursor), 7
	?call lcd_middle_7
	neiw LO(lcd_cursor), 8
	?call lcd_middle_8
	neiw LO(lcd_cursor), 10
	?call lcd_bottom_row
	neiw LO(lcd_cursor), 18
	?call lcd_middle_row
	ret

lcd_digit:
	gti A, 9
	?adi A, '0'      ! not greater 9, change into ascii '0'...'9'
	gti A, 16
	?adi A, 'A' - 10 ! not yet ascii, change into 'A'... 'F'
	call lcd_chr
	ret

lcd_hex:
	push VA
	slr A
	slr A
	slr A
	slr A
	call lcd_digit
	pop VA
	ani A, 0x0F
	call lcd_digit
	ret

! BC has column offset and count of columns

lcd_symbol_onoff:
	eqi A, 0
	?jmp lcd_symbol_on
	jmp lcd_symbol_off

lcd_symbol_on:
	call lcd_command_select_3
	mvi A, 0x80
	add A, B                ! datapointer
	call lcd_byte
	mvi A, 0x6A             ! - set OR mode
	call lcd_byte
	call lcd_data_select_3
1:
	mvi A, 0x80
	call lcd_byte
	dcr C
	?jr 1b

	jmp lcd_middle_row

lcd_symbol_off:
	call lcd_command_select_3
	mvi A, 0x80
	add A, B                ! datapointer
	call lcd_byte
	mvi A, 0x6E             ! - set AND mode
	call lcd_byte
	call lcd_data_select_3
1:
	mvi A, 0x7F
	call lcd_byte
	dcr C
	?jr 1b

	jmp lcd_middle_row

! 2 small cells at the top-right corner

lcd_top_row:

	call lcd_command_select_3
	mvi A, 0x72             ! - character mode, right entry
	call lcd_byte
	mvi A, 0x80 + 6 * 5     ! - datapointer. past the middle row 8 cells
	call lcd_byte
	call lcd_data_select_3

	mviw LO(lcd_cursor), 8  ! past the middle row 8 cells
	ret

! 8 large cells in the middle

lcd_middle_row:

	call lcd_command_select_3
	mvi A, 0x72             ! - character mode, right entry
	call lcd_byte
	mvi A, 0x80             ! - datapointer
	call lcd_byte
	call lcd_data_select_3

	mviw LO(lcd_cursor), 0
	ret

lcd_middle_3:
	call lcd_command_select_3
	mvi A, 0x72             ! - character mode, right entry
	call lcd_byte
	mvi A, 0x80 + 9 * 5     ! - datapointer
	call lcd_byte
	call lcd_data_select_3
	ret

lcd_middle_4:
	call lcd_command_select_3
	mvi A, 0x72             ! - character mode, right entry
	call lcd_byte
	mvi A, 0x80 + 3 * 5     ! - datapointer
	call lcd_byte
	call lcd_data_select_3
	ret

lcd_middle_7:
	call lcd_command_select_3
	mvi A, 0x72             ! - character mode, right entry
	call lcd_byte
	mvi A, 0x80 + 8 * 5     ! - datapointer
	call lcd_byte
	call lcd_data_select_3
	ret

lcd_middle_8:
	call lcd_command_select_3
	mvi A, 0x72             ! - character mode, right entry
	call lcd_byte
	mvi A, 0x80 + 6 * 5    ! - datapointer
	call lcd_byte
	call lcd_data_select_3
	ret

lcd_bottom_row:

	call lcd_command_select_0
	mvi A, 0x72             ! - character mode, right entry
	call lcd_byte
	mvi A, 0x80             ! - datapointer
	call lcd_byte
	call lcd_data_select_0

	mviw LO(lcd_cursor), 10
	ret

audio_icons:
        lxi BC, ICON_LEFT
        mov A, [mute_flags]
        ani A, 1
        call lcd_symbol_onoff

        lxi BC, ICON_RIGHT
        mov A, [mute_flags]
        ani A, 2
        call lcd_symbol_onoff

        ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  KBD
!
!  scancode 0 is nokey, 1...0x1C are valid, some holes
!  POWERSW@PB translated to 'Q'


scancodes:
	.byte 0

	.byte '+' ! 1C +
	.byte 'O' ! 02 OK (LOCK)
	.byte 'N' ! 03 NEXT (PREV)
	.byte 'P' ! 04 PTT

	.byte '-' ! 1B -
	.byte 'A' ! 06 R (ALPHA)
	.byte '?' ! 07 MENU (MFT)
	.byte 'R' ! 08 RCL (STO)

	.byte 0
	.byte 'M' ! 01 MUTE
	.byte 0
	.byte 0

	.byte '3' ! 09 3 (GHI)
	.byte '2' ! 0A 2 (DEF)
	.byte '1' ! 0B 1 (ABC)
	.byte 'F' ! 0C FCN

	.byte '6' ! 0D 6 (PQR)
	.byte '5' ! 0E 5 (MNO)
	.byte '4' ! 0F 4 (JKL)
	.byte 'C' ! 10 CLR (CLA)

	.byte '9' ! 11 9 (YZ-)
	.byte '8' ! 12 8 (VWX)
	.byte '7' ! 13 7 (STU)
	.byte 'E' ! 14 luuri     (Earphone)

	.byte '#' ! 15 #
	.byte '0' ! 16 0 (AoA:O:)
	.byte '*' ! 17 *
	.byte 'H' ! 18 HF

	.byte 0
	.byte 0
	.byte 0
	.byte 0

kbd_seq:
	.byte KBD_R0, KBD_R1, KBD_R2, KBD_R3, KBD_R4, KBD_R5, KBD_R6

kbd_init:
	lxi HL, kbd_seq
	ldax [HL]
	mov PA, A
	call kbd_wait

	mvi PA, SCK	! mask kbd_seq lines out
	call kbd_wait
	
	skit AN4
	?nop
	skit AN5
	?nop
	skit AN6
	?nop
	skit AN7
	?nop
	ret

scan_keys:
        call hookcheck

	mvi D, 0         ! Pick current character to here

        offi PB, POWERSW
        ?mviw LO(seen_pwr), 1 ! No, POWERSW is high.
        oni PB, POWERSW
        ?mvi D, 'Q'           ! No, POWERSW is pulled low.
        oniw LO(seen_pwr), 1
        ?mvi D, 0             ! No, has not been high yet.

	mvi B, 0         ! walk up scancode
	mvi C, 6         ! 7 lines
	lxi HL, kbd_seq
1:
	ldax [HL]+
	mov PA, A
	call kbd_wait

	mvi PA, SCK	! trailing edge on common rail, all zeros, SCK = 1
	call kbd_wait

	inr B            ! B will not overflow, scancodes 1...29
	?sknit AN4
	?call kbd_hit
	inr B
	?sknit AN5
	?call kbd_hit
	inr B
	?sknit AN6
	?call kbd_hit
	inr B
	?sknit AN7
	?call kbd_hit
	dcr C
	?jre 1b

	mviw LO(kbd_bounce), 10 ! next stop here

	mov A, D
	neaw LO(cur_key)
	?ret
	eqi A, 0
	?jmp got_key            ! no key into some key
	jmp no_key              ! some key into none

kbd_hit:
	push HL
	lxi HL, scancodes
	ldax [HL+B]
	mov D, A
	pop HL
	ret

!----------------------------------------------------------------------
!
!  Release of a key (none currently pressed)
!
no_key:
	mviw LO(kbd_repeat), 0
	call dtmf_off

        ldaw LO(cur_key)
        mvi B, 'B'
        call mbus_sendkey

        mvi A, 0
        staw LO(cur_key)        ! state changed, remember
	ret

!----------------------------------------------------------------------
!
!  New keyboard press, character in A
!
got_key:
        staw LO(cur_key)        ! remember it

	mviw LO(kbd_repeat), 255 - 112 ! typematic after 1 sec

        oniw LO(hsic_led), LED_ROAM  ! ROAM is on.
	?call keyclick

        offiw LO(hsic_led), LED_ROAM  ! ROAM is off.
	?call dtmf_kbd

	mov A, [cur_key]
	mvi B, 'M'
	call mbus_sendkey

	ret

!----------------------------------------------------------------------
!
!  Typematic keyboard action, character in cur_key
!
repeat_key:
	mviw LO(kbd_repeat), 255 - 112 / 3 ! typematic rate 3 Hz

        oniw LO(hsic_led), LED_ROAM  ! ROAM is on.
	?call keyclick

	mov A, [cur_key]
	mvi B, 'R'
	call mbus_sendkey

	ret

kbd_wait:
	nop
	nop
	nop
	nop
	nop
	nop
	ret

hookcheck:
        mov B, [last_pc]
        mov A, PC
        mov [last_pc], A
        xra A, B
        oni A, OFFHOOK
        ?ret

        mov B, [last_pc]
        mvi A, 'f'
        oni B, OFFHOOK
        ?mvi A, 'n'

        mvi B, 'H'
        call mbus_sendkey

        call select_mic
        ret

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!
!  MBUS
!
#include "mbus.asm" /* again to get the text part */

key_proto: .ascii "ANY1", MBUSID, "M*\4"
!                  0123   4567     89 0

mbus_sendkey:
	lxi HL, key_proto
	lxi DE, mbus_txmsg
	mvi C, SIZE(key_proto) - 1
	block
	lxi HL, mbus_txmsg
	stax [HL + 9]
	mov A, B
	stax [HL + 8]

	jmp mbus_send

!----------------------------------------------------------------------
mbus_newmsg:

	ldax [HL]+
	eqi A, 'H'
	?ret
	ldax [HL]+
	eqi A, 'S'
	?ret
	ldax [HL]+
	eqi A, 'N'
	?ret
	ldax [HL]+
	eqi A, '2'
	?ret

 	mvi A, LED_CALL          ! ours. dim out 'roaming'. bah.
 	call latch_0

	ldax [HL]+               ! sender at [HL] now
	nei A, 0xFF
	?ret
	ldax [HL]+
	nei A, 0xFF
	?ret
	ldax [HL]+
	nei A, 0xFF
	?ret
	ldax [HL]+
	nei A, 0xFF
	?ret

	ldax [HL]+     ! now after addresses, length at C, \4 not included

	nei A, 'd'
	?jmp msg_display
	nei A, 'f'
	?jmp msg_freq
	nei A, 's'
	?jmp msg_signal_report
	nei A, 'v'
        ?jmp msg_volume
        nei A, 'm'
        ?jmp msg_mute
        nei A, 'l'
	?jmp msg_leds
	nei A, 'b'
	?jmp msg_beeps
	nei A, 'h'		!!
	?jmp msg_memory		!!

	ret ! hmph. dunno what it is.

!----------------------------------------------------------------------
! HL points to message data after command designator character.
! Length of data in C
! e.g. HL->"433500", C=6

!    0 "HSN2" "...." "d" "........" \4  e.g. display "hepokatt"

msg_display:
	call lcd_middle_row
	lti C, 8
	?mvi C, 8            ! truncate text to 8
	push BC
	call lcd_block       ! leftjust
	pop BC
	mvi A, 8
	sub A, C
	lxi HL, blanks
	mov C, A
	call lcd_block       ! pad right
	ret

!    0 "HSN2" "...." "f" "........" \4  e.g. display "  433500"

msg_freq:
	call lcd_bottom_row
	lti C, 8
	?mvi C, 8            ! truncate text to 8
	mvi A, 8
	sub A, C
	push BC
	push HL
	lxi HL, blanks       ! pad left
	mov C, A
	call lcd_block
	pop HL
	pop BC
	call lcd_block       ! rightjust
	ret

!    0 "HSN2" "...." "s" "..." \4  e.g. display "99" and light/dim ON-AIR

msg_signal_report:
	push HL
	push BC
	call lcd_top_row
	lti C, 2
	?mvi C, 2            ! truncate text to 2
	mvi A, 2
	sub A, C
	push BC
	push HL
	lxi HL, blanks       ! pad left
	mov C, A
	call lcd_block
	pop HL
	pop BC
	call lcd_block       ! rightjust
	pop BC
	pop HL
	gti C, 2
	?ret
	ldax [HL+2]          ! we have ON-AIR/SERV too
	nei A, 'T'
	?jr 2f
	nei A, 'S'
	?jr 1f
	mvi A, LED_SERV      ! nor 'T' or 'S', dim both
	call latch_0
	mvi A, LED_ROAM
	call latch_0
	ret
1:
	mvi A, LED_SERV      ! 'S', SERV
	call latch_1
	mvi A, LED_ROAM
	call latch_0
	ret
2:
	mvi A, LED_SERV      ! 'T', ON-AIR
	call latch_0
	mvi A, LED_ROAM
	call latch_1
	ret

!    0 "HSN2" "...." "v" "." \4         e.g. volume setting 0..9

msg_volume:
	ldax [HL]+
	sui A, '0'
	lti A, 9
	?mvi A, 9
	mov [volume], A
	jmp vol_set

!    0 "HSN2" "...." "m" "s?e?" \4      e.g. mute spkr and/or earphone

msg_mute:
	mvi B, 0
	jr 2f
1:
	ldax [HL]+
	nei A, 'e'
	?ori B, 1
	nei A, 's'
	?ori B, 2
2:
	dcr C
	?jr 1b
	mov [mute_flags], B
	call audio_icons
	jmp mute_set

!    0 "HSN2" "...." "b" ".*" \4        e.g. beeps

msg_beeps:
	ldax [HL]+
	push HL
	push BC
	call 1f
	pop BC
	pop HL
	dcr C
	?jmp msg_beeps

	call dtmf_off		! switch off dtmf

	ret
1:
	call dtmf_chr
	call nap
	ret
nap:
	hlt; hlt; hlt
	hlt; hlt; hlt
	hlt; hlt; hlt
	hlt
	ret

!    0 "HSN2" "...." "l" "sT" \4        e.g. markers SERV=off, ROAM=on

msg_leds:
	ldax [HL]+
	push HL
	push BC
	call 1f
	pop BC
	pop HL
	dcr C
	?jmp msg_leds
	ret
1:
	mov B, A              ! save aBoriginal
	mov C, A
	ani C, ~0x20          ! Capitalize it for table
	lxi HL, led_chars - 1
1:
	inx HL
	ldax [HL]+
	nei A, 0      ! not last.
	?jr 1f
	eqa A, C      ! same.
	?jr 1b
	jmp 3f        ! hit. 
1:
	lxi HL, lcd_chars - 2
1:
	inx HL        ! words in table
	inx HL
	ldax [HL]+
	nei A, 0      ! not last.
	?jr 1f
	eqa A, C      ! same.
	?jr 1b
	jmp 4f        ! hit.
1:
	ret           ! unknown symbol. ignore.
3:
	ldax [HL]
 	offi B, 0x20  ! uppercase lights the led.
 	?jmp latch_0
 	jmp latch_1
4:
	ldeax [HL]
	mov A, B
	ani A, 0x20   ! extract case-bit
	xri A, 0x20   ! flip, uppercase lights the icon
	dmov BC, EA
	jmp lcd_symbol_onoff

led_chars:
	.byte 'O', LED_ON
	.byte 'S', LED_SERV
	.byte 'T', LED_ROAM     ! T for TX
	.byte 'H', LED_HF
	.byte 'M', LED_MFT
	.byte 'C', LED_CALL
	.byte 'B', BACKLIGHT
	.byte 0
lcd_chars:
	.byte 'F'; .word ICON_FCN
	.byte 'A'; .word ICON_ALPHA
	.byte 'K'; .word ICON_KEY
	.byte 'D'; .word ICON_T		! tone icon
	.byte 'L'; .word ICON_LEFT
	.byte 'R'; .word ICON_RIGHT
	.byte 'N'; .word ICON_MINUS
	.byte 0


!    0 "HSN2" "...." "h" "xyy" \4         e.g. hsic port #x feed with yy

msg_memory:
	ldax [HL]+
	sui A, '0'
	lti A, 3
	? mvi A, 3
	mov B, A 

	ldax [HL]+
	sui A, '0'	! subtract ascii '0'
	lti A, 10
	? sui A, 'A'-'9'+1 ! should be 7
	sll A
	sll A
	sll A
	sll A
	ani A, 0xF0
	mov C, A

	ldax [HL]+
	sui A, '0'	! subtract ascii '0'
	lti A, 10
	? sui A, 'A'-'9'-1 ! should be 7
	ani A, 0x0F
	ora A, C	! add all together
	lxi HL, hsic0	! HSIC reg. mirror's base address
	stax [HL+B]	! feed HSIC now

	ret


!----------------------------------------------------------------------
hsic_update:

	push BC
	mov C, [hsic0]
	mov B, [hsic1]
	mov A, [hsic2]
	
	offi A, SPKMODE				! DTMF or RX audio for ESPKR? (SPKMODE masters)
	?jr 1f
	ori B, SOMEMUTE			! mute some audio for DTMF to be heard
	jr 2f
1:	offi B, SPKRXMUTE 				! 
	?jr 2f
	ori B, SOMEMUTE

2:	offi A, MODMODE					! DTMF of any other audio selected for TX (modulation)
	?jr 1f
	ori C, IMICMUTE | EXTMICMUTE | HFMICMUTE 	! mute all, DTMF is active now (MODMODE masters muting)
	jr 3f						! skip the rest
1:	offi C, IMICMUTE 				! following mutes the other sources
	?jr 2f
	ori C, EXTMICMUTE | HFMICMUTE 
	jr 3f
2:	offi C, EXTMICMUTE
	?jr 3f
	ori C, HFMICMUTE 
	
3:	mov [HSIC_ADDR0], C
	mov [HSIC_ADDR1], B
	mov [HSIC_ADDR2], A
	mov A, [hsic_led]
	mov [HSIC_ADDR3], A
	pop BC
	ret

!----------------------------------------------------------------------

	.ascii "@(#)"
version:
	.asciz VERSION
TheEnd:

